// ALGORITHM.CPP
//
// Selection sort, binary search, and
//   Euclid's algorithm for GCF.

#include <iostream.h>
#include "apvector.h"

void Sort(apvector<double> &a)

// Sort a[0], ..., a[size-1] in ascending order.

{
    int i, iMax, n;
    double aTemp;
    int size = a.length();

    for (n = size;   n >= 2;   n--) {

        // Find the index "iMax" of the largest element
        //   among a[0], ..., a[n-1]:

        iMax = 0;
        for (i = 1;   i < n;   i++)
            if (a[i] > a[iMax])
                iMax = i;

        // Swap a[iMax] with a[n-1]:

        aTemp = a[iMax];    // Save a[iMax] in a temporary location.
        a[iMax] = a[n-1];   // Copy a[n-1] to a[iMax].
        a[n-1] = aTemp;     // Copy saved value to a[n-1].

        // Decrement n (accomplished by n-- in the "for" loop).
    }
}

//****************************************************************

int Search(const apvector<int> &a, int target)

// Performs binary search on the array
//   a[0] < a[1] < ... < a[size-1].
//   Looks for an element a[k] equal to "target".
// Returns k if the target is found; -1 otherwise.

{
    int left = 0, right = a.length() - 1, middle;
    int k = -1;

    while (left <= right) {

        // Take the index of the middle element between
        //   "left" and "right":

        middle = (left + right) / 2;

        // Compare  this element to the "target" value
        //  and adjust the search range accordingly:

        if (target > a[middle])
            left = middle + 1;
        else if (target < a[middle])
            right = middle - 1;
        else {     // target must be equal to a[middle]
            k = middle;
            break;
        }
    }

    return k;
}

//****************************************************************

int GCF(int m, int n)

// Returns the greatest common factor of two positive integers
//   m and n.

{
    int r;

    for(;;) {  // Repeat (until break)
        r = m % n;         // r = "m modulo n" i.e. the remainder
                           //    when m is divided by n.
        if (r == 0)        // i.e. if n evenly divides m
             break;
        m = n;             // Substitute values: m becomes n and 
        n = r;             //   n becomes r.  GCF(m,n) = GCF(n,r).
    }
    return n;
}

//****************************************************************

int main()

{
    apvector<double> x(6);
    x[0] = 1.1; x[1] = 3.3; x[2] = 5.5;
    x[3] = 6.6; x[4] = 4.4; x[5] = 2.2;

    Sort(x);
    cout << "Sorted: " << x[0] << ' ' << x[1] << ' ' << x[2] << ' '
                       << x[3] << ' ' << x[4] << ' ' << x[5] << endl;
    apvector<int> a(6);
    a[0] =  8; a[1] = 13; a[2] = 21;
    a[3] = 34, a[4] = 55; a[5] = 89;
    int target = 34;

    cout << "Search: position of " << target << " = "
                   << Search(a, target) << endl;
 

    int m = 1001, n= 4235;
    cout << "GCF(" << m << ',' << n << ") = " << GCF(m,n) << endl;

    return 0;
}
