/* DICT.CPP

    This program works as an online dictionary.  It reads a word
    and shows its translation.  The program loads the dictionary
    data from the DICT.DAT file.
 
    Author: M. Webster
    Rev. 1.0ap  09/20/98
*/

#include <iostream.h>
#include <fstream.h>
#include "bool.h"
#include "apvector.h"
#include "apstring.h"

struct ENTRY {
    apstring word;
    apstring translation;
};

const int MAXWORDS = 1000;  // Max number of words in the dictionary

// Function prototypes:
bool LoadDictionary(apstring fileName, apvector<ENTRY> &dict);
bool FoundWord(const apvector<ENTRY> &dict,
               const apstring &word, apstring &translation);

/****************************************************************/
/**************            Main Program             *************/
/****************************************************************/

int main()

{
    apvector<ENTRY> dict(MAXWORDS);
    apstring word, translation;
    bool ok, quit;

    // Load the dictionary from the file

    ok = LoadDictionary("DICT.DAT", dict);
    if (!ok) {
        cout << "*** Cannot load dictionary ***\n";
        return 1;
    }

    // Translate words:

    quit = false;
    while (!quit) {
        cout << "Enter a word or 'q' to quit ==> ";
        cin >> word;             // Read one word and
        cin.ignore(80, '\n');    //   skip the rest of the line
        if (word == "q")
            quit = true;
        else if (FoundWord(dict, word, translation))
            cout << translation << "\n\n";
        else
            cout << word << " -- not in the dictionary.\n\n";
    }
    return 0;
}

/****************************************************************/
/**************              Functions              *************/
/****************************************************************/

bool LoadDictionary(apstring fileName, apvector<ENTRY> &dict)

// Reads dictionary entries from a file.
// Returns true if successful, false if cannot open the file.

{
    int cnt = 0;
    apstring line;

    // Open dictionary file:
    ifstream inpFile(fileName.c_str());
    if (!inpFile)
        return false;

    // Read and display the header line:
    getline(inpFile, line);
    cout << line << endl;

    // Read words and translations into the dictionary array:
    while (cnt < MAXWORDS &&
           inpFile >> dict[cnt].word >> dict[cnt].translation) {
        inpFile.ignore(80, '\n'); // Skip the rest of the line
        cnt++;
    }

    // Report the number of entries:
    cout << " (" << cnt << " words)\n\n";
    dict.resize(cnt);

    return true;
}

/****************************************************************/

bool FoundWord(const apvector<ENTRY> &dict,
               const apstring &word, apstring &translation)

// Finds a word in the dictionary.
//  dict -- the dictionary array
//  word -- word to translate
//  translation -- returned translation from the dictionary.
// Returns true if the word has been found, false otherwise.

{
    bool found = false;
    int i, len = dict.length();

    for (i = 0;   !found && i < len;   i++) {
        if (dict[i].word == word) {
            translation = dict[i].translation;
            found = true;
        }
    }
    return found;
}
