// GRADES.CPP
//
// This program reads students' grades from a file, GRADES.DAT,
//   and finds the average, the highest and the lowest grade.
//
// Author: Mark Avgerinos
// Date 05-21-1999
//

#include <iostream.h>
#include <fstream.h>  // Supports file I/O
#include <iomanip.h>
#include "bool.h"
#include "apvector.h"

//****************************************************************
//******************    Function Prototypes    *******************
//****************************************************************

bool ReadNumbers (char fileName[], apvector<int> &a);
int AddElements (const apvector<int> &a);
int MaxElement (const apvector<int> &a);
int MinElement (const apvector<int> &a);

//****************************************************************
//******************            Main           *******************
//****************************************************************

int main()

{
    const int MAXSTUDENTS = 100;
    int nStudents;
    apvector<int> grades(MAXSTUDENTS);
    double avgGrade;

    if (!ReadNumbers("GRADES.DAT", grades)) {
        cout << "Cannot open GRADES.DAT.\n";
        return 1;
    }

    nStudents = grades.length();
    if (nStudents == 0) {
        cout << "GRADES.DAT file contains no numbers.\n";
        return 1;
    }

    cout << nStudents << " students\n";

    avgGrade = double(AddElements(grades)) / nStudents;
    cout.setf(ios::showpoint | ios::fixed);
    cout << "The average grade is "
         << setprecision(1) << avgGrade << endl;
    cout << "The highest grade is "
         << MaxElement(grades) << endl;
    cout << "The lowest grade is "
         << MinElement(grades) << endl;

    return 0;
}

//****************************************************************
//******************          Functions        *******************
//****************************************************************

bool ReadNumbers (char fileName[], apvector<int> &a)

// Reads integers from a file "fileName" into the array a.
// Returns true if the file is opened successfully, false otherwise.

{
    int n = 0, len = a.length();

    // Declare a variable "inpFile" of the data type "ifstream"
    //   -- input file stream -- and open the file with the
    //   requested name "fileName" for reading:

    ifstream inpFile(fileName);

    // Checks that the file exists:

    if (!inpFile)
        return false;

    // Extract numbers from the file input stream
    //   (i.e. read numbers from the file into the array)
    //   until the end of file, or array is full:

    while (n < len && inpFile >> a[n])
        n++;

    // Resize the array to the actual number of grades read:
    a.resize(n);

    return true;
}

//****************************************************************

int AddElements (const apvector<int> &a)

// Returns a[0] + a[1] + ... + a[len-1]

{
    ...
    ...
}

//****************************************************************

int MaxElement (const apvector<int> &a)

// Returns the value of the largest element of the array a.

{
    ...
    ...
}

//****************************************************************

int MinElement (const apvector<int> &a)

// Returns the value of the smallest element of the array a.

{
    ...
    ...
}

