// WEEKDAY.CPP
//
//   This program calculates the day on which the
//   user's birthday (or any other date) falls.
//
//   Author: Cal Lenders  
//   Rev 1.0
//

#include <iostream.h>
#include "bool.h"

//****************************************************************
//*************      Tables and functions      *******************
//****************************************************************

bool LeapYear (int year)

// "year" must be between 1900 and 2999.
// Returns true if "year" is a leap year.

{
        //  true, if year is divisible by 4, and ...
        //    ... either not divisible by 100, or divisible by 400. 

    return (year % 4 == 0 &&
                (year % 100 != 0 || year % 400 == 0));
}

//****************************************************************

const int daysInMonth[12] = {

    31, 28, 31, 30, 31, 30,
    31, 31, 30, 31, 30, 31
};

// Days from the beginning of the year
//   to the beginning of the month:

const int daysToMonth[12] = {
     0,  31,  59,  90, 120, 151,
   181, 212, 243, 273, 304, 334
};

const char *dayName[7] = {
    "Sunday",
    "Monday",
    "Tuesday",
    "Wednesday",
    "Thursday",
    "Friday",
    "Saturday"
};

const int DAY0 = 1; // Day of week for 01-01-1900 is Monday = 1.

//****************************************************************

bool ValidDate (int month, int day, int year)

// Returns true if month-day-year is a valid date between
//   01-01-1900 and 12-31-2999.

{
    bool valid = false;  // First assume the date is invalid.
    int days;

    // If year and month have valid values:
    if (year >= 1900 && year <= 2999 &&
                                month >= 1 && month <= 12) {

        // Get the number of days in this month from the table:
        days = daysInMonth[month-1]; // (-1, because the indices
                                     //   of an array start from 0.)

        // If February of a leap year -- increment the number
        //   of days in this month:
        if (month == 2 && LeapYear(year))
            days++;

        // Check that the given day is within the range.
        //   If so, set valid to true:
        if (day >= 1 && day <= days)
            valid = true;
    }

    return valid;
}

//****************************************************************

long DaysSince1900 (int month, int day, int year)

// Returns the number of days elapsed since 01-01-1900
//  to month-day-year

{
    long days;

    // Calculate days to 01-01 of this year with correction for
    //   all the previous leap years:
    if (year == 1900)
        days = 0;
    else
        days = long(year - 1900) * 365
           + (year - 1901) / 4       // +1 for each 4th year
           - (year - 1901) / 100     // -1 for each 100th year
           + (year - 1601) / 400;    // +1 for each 400th year,
                                     //     starting at 2000

    // Add days for previous months with correction for
    //   the current leap year:
    days += daysToMonth[month-1];
    if (LeapYear(year) && month > 2) days++;

    // Add days since the beginning of the month:
    days += day - 1;

    return days;
}

//****************************************************************

int DayOfWeek (int month, int day, int year)

// Returns the day of the week for a given date:
//   0 -- Sunday, 1 -- Monday, etc.

{
    return int((DAY0 + DaysSince1900(month, day, year)) % 7);
}

//****************************************************************
//*************                main            *******************
//****************************************************************

int main()

{
    int month, day, year;
    int weekday;

    cout << "Please enter a date (e.g. 11 23 2001) ==> ";
    cin >> month >> day >> year;

    if (!ValidDate(month, day, year)) {
        cout << "*** Invalid date ***\n";
        return 1;
    }

    weekday = DayOfWeek(month, day, year);

    // Display the entered date and the name of the calculated
    //  day of week:
    cout << month << '-' << day << '-' << year << ", "
         << dayName[weekday] << endl;

    return 0;
}

