// EMAIL.CPP
//
//   This program sets up a toy e-mail system.
//
//   The program maintains a list of subscribers in an array.
//   Initially the list is empty.  The program adds new
//   subscribers automatically when a new name "logs in" and
//   when a message is addressed to a new name.
//   The list may have at most 10 subscribers.
//
//   Author: Eudora Scanty

#include <iostream.h>
#include <ctype.h>
#include <string.h>
#include "apstring.h"
#include "apqueue.h"

struct MESSAGE {
    apstring from;
    apstring to;
    apstring subject;
    apstring text;
};

struct SUBSCRIBER {
    apstring name;
    apqueue<MESSAGE> mailbox;
};

// Array of subscribers
//  (these constants and variables are declared as global because
//   they are central to this program and most of the functions
//   need them):

static const int MAXUSERS = 10;  // Maximum number of subscribers
static apvector<SUBSCRIBER> subscriber(MAXUSERS);
static int nUsers = 0;    // Current number of users

//****************************************************************

static int FindUser(const apstring &name)

// Returns the index of the user whose name matches
//   "name", or -1 if not found.

{
    ...
    ...
}

//****************************************************************

static int NewUser(const apstring &name)

// Adds a new user to the array of subscribers.
// Returns the index of the new subscriber or
//   -1 if failed.

{
    int n;

    if (nUsers >= MAXUSERS) {
        cout << "Cannot add " << name << " (out of space).\n";
        return -1;
    }
    ...
    ...
}

//****************************************************************

static int Login(const apstring &name)

// Finds the name in the subscriber array or
//   adds a new user with the given name if not found.
// Returns the index of logged-in user,
//   or -1 if failed.

{
    int n;

    n = FindUser(name);
    if (n < 0)
        n = NewUser(name);

    return n;
}

//****************************************************************

static void ProcessCommands(int n)

{
    char key = ' ';
    apstring name;
    MESSAGE msg;
    int nDest;   // index of the destination subscriber.

    while (key != 'Q') {

        // Show menu, get next command:

        cout << "     (R)ead next message\n";
        cout << "     (S)end message\n";        
        cout << "     (Q)uit\n";
        cout << "\n";
        cout << "Next command ==> ";
        cin >> key;
        cin.ignore(256, '\n');
        key = toupper(key);

        // Execute command:

        switch (key) {

          case 'R':    // Read next message

            ...
            ...

            break;

          case 'S':    // Send message


            ...
            ...

            break;

          case 'Q':        // Quit
            break;

          default:            
            cout << "*** Invalid command ***\n";
            break;
        }
    }
}

//****************************************************************

int main()

{
    apstring name;
    int n;

    for (;;) {
        cout << "Username (or 'Q' to quit): ";
        getline(cin, name);
        if (name == "Q" || name == "q")
            break;
        n = Login(name);
        if (n >= 0) {
            cout << "\nWelcome " << subscriber[n].name << "\n\n";
            ProcessCommands(n);
        }
    }

    return 0;
}

