// LIST.H
//=======

#include "bool.h"
#include "apstring.h"

struct NODE {
    apstring data;
    NODE *next;
};

// *** Linked List class ***

class LIST {

  private:

    NODE *head;
    NODE *tail;

  public:

    LIST();
    void Insert (const apstring &str);
    ~LIST();

  friend class LISTITER;

};

// *** Companion iterator class ***

class LISTITER {

  private:

    NODE *current;

  public:

    LISTITER(const LIST &list);         // Constructor from a LIST object
    LISTITER(const LISTITER &listiter); // Copy constructor

    bool NextData(apstring &str);
            // Places data from the current node into str and
            //   advances the current pointer.
            // Returns true if successful, false if at the end of the list.
};


// LIST.CPP
// ========

// #include "list.h"

// *** LIST implementation: ***

LIST::LIST()

// Constructor

        : head(0), tail(0)
{}

void LIST::Insert (const apstring &str)

// Inserts str into the list.

{
    NODE *newNode;
    newNode = new NODE;
    newNode->data = str;
    newNode->next = 0;
    if (tail)
        tail->next = newNode;
    else
        head = newNode;
    tail = newNode;
}

LIST::~LIST()

// Destructor: deletes all nodes

{
    NODE *temp;

    while (head) {
        temp = head->next;
        delete head;
        head = temp;
    }
}

// *** LISTITER implementation ***

LISTITER::LISTITER(const LIST &list)

// Constructor from LIST

{
    current = list.head;
}

LISTITER::LISTITER(const LISTITER &otheriter)

// Copy constructor

{
    current = otheriter.current;
    // This is redundant: we could use the automatic member-by-member
    //   copy constructor.  Here we intentionally want the pointer
    //   current in the original iterator and in the copy to point
    //   to the same memory location--a node in the list.
}

bool LISTITER::NextData(apstring &str)   

// Places data from the current node into str and
//   advances the current pointer.
// Returns true if successful, false if at the end of the list.

{
    if (!current)
        return false;

    str = current->data;
    current = current->next;

    return true;
}

// TEST.CPP
// ========

#include <iostream.h>
//#include "apstring.h"
//#include "list.h"

void Print(LIST &list)

// Prints all elements in the list using one iterator.

{
    LISTITER listiter(list);
    apstring str;

    while (listiter.NextData(str))
        cout << str << ' ';
    cout << endl;    
}

//****************************************************************

void PrintDuplicates(LIST &list)

// Prints all elements in the list that have duplicates
//   by using two nested iterators.

{
    cout << "Duplicates:\n";

    LISTITER listiter1(list);
    apstring str1, str2;

    while (listiter1.NextData(str1)) {
        LISTITER listiter2 = listiter1;
        while (listiter2.NextData(str2)) {
            if (str1 == str2) {
                cout << '[' << str1 << ']' << endl;
                break;     // Break from the inner while loop.
            }
        }
    }
}

//****************************************************************

int main()

{
    LIST list;

    list.Insert("a");  // Automatic cast from const char * to
                       // apstring.  Calls the apstring(const char *)
                       // constructor.
    list.Insert("rose");
    list.Insert("is");
    list.Insert("a");
    list.Insert("rose");
    Print(list);
    cout << endl;

    PrintDuplicates(list);
    return 0;
}

