// LIST.CPP
//
// This program implements and tests linked list functions.
//   The program builds a linked list from the names of cities,
//   stored in the LIST.DAT file.  It then displays a menu
//   to insert and remove elements from the list, and
//   other commands.
//
// Author: Taylor Weaklink
//

#include <iostream.h>
#include <fstream.h> 
#include <ctype.h>        // Declares toupper(ch)
#include "apstring.h"

struct NODE {
    apstring city;
    NODE *next;
};

enum STATUS {FAILED, OK};

// Function prototypes:

STATUS LoadList (NODE* &head, const apstring &fileName);
void DisplayList (NODE *head);
STATUS Insert (NODE* &head, const apstring &city);
STATUS InsertInOrder (NODE* &head, const apstring &city);
STATUS Remove (NODE* &head, const apstring &city);
void DestroyList (NODE *head);

#ifdef EXTRA_CREDIT
STATUS SaveList (NODE *head, const apstring &fileName);
#endif

//****************************************************************
//**************       Linked List functions       ***************
//****************************************************************

STATUS LoadList (NODE* &head, const apstring &fileName)

// Loads names from file "fileName" and inserts them
//   into a linked list's tail.
// Returns OK if successful, FAILED if cannot open the file.

{
    ...
    ...
}

//****************************************************************

#ifdef EXTRA_CREDIT

STATUS SaveList (NODE *head, const apstring &fileName)

// Saves names from the list in "fileName".
// Returns OK if successful, FAILED if cannot open the file.

{
    ...
    ...
}

#endif

//****************************************************************

void DisplayList (NODE *head)

// Prints out the names from the list.

{
    ...
    ...
}

//****************************************************************

STATUS Insert (NODE* &head, const apstring &city)

// Inserts "city" at the head of the linked list.
// Returns OK if successful, FAILED if could not
//   allocate a new node.

{
    NODE *newnode;

    // 1. Allocate a new node:

    newnode = new NODE;
    if (!newnode)
        return FAILED;      // Out of memory.

    // 2. Copy the information into the new node:

    newnode->city = city;

    // 3. Link the new node to the list:

    newnode->next = head;   // Append the old list to newnode.
    head = newnode;         // Change the head to newnode.

    return OK;
}

//****************************************************************

STATUS InsertInOrder (NODE* &head, const apstring &city)

// Inserts "city" in alphabetical order into the linked list.
//   Assumes that the list is arranged in alphabetical order.
//   Duplicate names are allowed.
// Returns OK if successful, FAILED if could not
//   allocate a new node.

{
    NODE *newnode;

    // 1. Allocate a new node:

    newnode = new NODE;
    if (!newnode)
        return FAILED;

    // 2. Copy the information into newnode:

    newnode->city = city;

    // 3. Link newnode to the list:

    //    3.1. Find the right place to insert newnode --
    //           between "prev" and "node":

    NODE *node = head, *prev = 0;

    while (node && node->city <= city) {
        prev = node;              // ... advance node and prev
        node = node->next;
    }

    //    3.2. Link newnode between "prev" and "node":

    newnode->next = node;       // Append "node" to newnode. 
    if (prev)
        prev->next = newnode;   // Insert after "prev".
    else
        head = newnode;         // No prev -- make newnode the
                                //   new head.
    return OK;
}

//****************************************************************

STATUS Remove (NODE* &head, const apstring &city)

// Removes "city" from the list.
// Returns OK if successful, FAILED if not found.

{
    NODE *node, *prev;

    // 1. Find and unlink the node:

    //    1.1. Find the node to remove.  Keep track of
    //           the previous node:

    prev = 0;
    node = head;

    while (node && node->city != city) {
        prev = node;
        node = node->next;
    }

    if (!node)
        return FAILED;     // The target value not found in the list.

    //    1.2. Unlink the node:

    if (prev)
        prev->next = node->next;
    else
        head = node->next;

    // 2. Deallocate the node and release its memory:

    delete node;

    return OK;
}

//****************************************************************

void DestroyList (NODE *head)

{
    ...
    ...
}

//****************************************************************
//****************             main            *******************
//****************************************************************

int main()

{
    ...
    ...
}

//****************************************************************

