// MORSE.CPP

#include <ctype.h>
#include "morse.h"

static const char DOT = '.';
static const char DASH = '-';

LNODE *TELEGRAPH::mHead = 0;
TNODE *TELEGRAPH::mRoot = 0;

//****************************************************************

void TELEGRAPH::Open()

// Builds the encoding list and decoding tree for TELEGRAPH.

{
    // Create the mHead node:
    mHead = new LNODE(' ', "");   // A space char is encoded as
                                  //   the empty string.

    // Create the mRoot node:

    mRoot = new TNODE;
    if (!mRoot) return;
    mRoot->symbol = ' ';

    // Add codes to the encoding list and decoding tree:

    Insert ('A', ".-");
    Insert ('B', "-...");
    Insert ('C', "-.-.");
    Insert ('D', "-..");
    Insert ('E', ".");
    Insert ('F', "..-.");
    Insert ('G', "--.");
    Insert ('H', "....");
    Insert ('I', "..");
    Insert ('J', ".---");
    Insert ('K', "-.-");
    Insert ('L', ".-..");
    Insert ('M', "--");
    Insert ('N', "-.");
    Insert ('O', "---");
    Insert ('P', ".--.");
    Insert ('Q', "--.-");
    Insert ('R', ".-.");
    Insert ('S', "...");
    Insert ('T', "-");
    Insert ('U', "..-");
    Insert ('V', "...-");
    Insert ('W', ".--");
    Insert ('X', "-..-");
    Insert ('Y', "-.--");
    Insert ('Z', "--..");
    Insert ('0', "-----");
    Insert ('1', ".----");
    Insert ('2', "..---");
    Insert ('3', "...--");
    Insert ('4', "....-");
    Insert ('5', ".....");
    Insert ('6', "-....");
    Insert ('7', "--...");
    Insert ('8', "---..");
    Insert ('9', "----.");
    Insert ('.', ".-.-.-");
    Insert (',', "--..--");
    Insert ('?', "..--..");
}

void TELEGRAPH::Close()

// Destroys the encoding list.
// Destroys the decoding tree by calling the recursive helper function.

{
    // Destroy the encoding list:
    ...
    ...

    // Destroy the decoding tree:
    RecursiveDestroy(mRoot);
    mRoot = 0;
}

//****************************************************************

void TELEGRAPH::RecursiveDestroy(TNODE *root)

{
    ...
    ...
}

void TELEGRAPH::Insert (char symbol, const apstring &morsecode)

// Inserts a symbol and its Morse code into the encoding list
//   and decoding tree.
//   Each Morse code character will correspond to a path in the tree
//   from the root to a node: at a "dot" go left, at a "dash" go
//   right.  The node at the end of the path holds the symbol
//   for that character.

{
    // Append a new node at the end of the encoding list:

    LNODE *lnode;

    lnode = mHead;
    while (lnode->next)
        lnode = lnode->next;
    lnode->next = new LNODE(symbol, morsecode);
                      
    // Insert a new node into the decoding tree:

    int i, len = morsecode.length();
    TNODE *tnode = mRoot;

    for (i = 0;   i < len;   i++) {
        if (morsecode[i] == DOT) {
            ...
            ...
        }
        else { // if (morsecode[i] == DASH)
            ...
            ...
        }
    }
    tnode->symbol = symbol;
}

//****************************************************************

void TELEGRAPH::Encode(const apstring &text, apstring &morse)

// Converts text into Morse code.

{
    int i, len = text.length();
    LNODE *lnode;
    char ch;

    morse = "";                        // Make the morse string empty;

    for (i = 0;   i < len;   i++) {
        ch = toupper(text[i]);

        // Find this symbol in the encoding list;
        //   (skip this symbol if not found):
        for (lnode = mHead;   lnode;   lnode = lnode->next)
            if (lnode->symbol == ch)
                break;

        // Append the code to the morse string
        //   (add one space to separate letters):
        if (lnode)
            morse += lnode->code + ' ';
/*
    The above statement looks neat, but it may be atrociously
    inefficient.  We start with an empty string and may end up
    with a string of considerable length.  Each time += is executed,
    the string buffer may be reallocated and the whole string will
    be copied into the new buffer.  This is repeated for every
    character in the text string.  To avoid such situations we
    could add a constructor to the apstring class that would
    build an empty string but allocate a large buffer for future
    expansion.  We could also add the space character to
    the codes that we place into the encoding list in the Insert()
    function instead of executing + every time here:

    (e.g. lnode->next = new LNODE(symbol, morsecode + ' ');)

    Alternatively, we could use null-terminated strings to handle
    concatenation more efficiently.
*/
    }
}

void TELEGRAPH::Decode(const apstring &morse, apstring &text)

// Converts Morse code into text.

{
    TNODE *tnode;
    int i, len = morse.length();
            
    text = "";                        // Make the text string empty;

    tnode = mRoot;

    // For each char in the encoded message (can be
    //   a dot, a dash, or a space):

    for (i = 0;   i < len;   i++) {
        // Go down the tree as long as you get dots or dashes.
        //   When you encounter a space, take the symbol from
        //   the node, add it to the text string, and return
        // to the mRoot.
            ...
            ...
        }
    }
}

