// MORSE.H

#ifndef _MORSE_H_
#define _MORSE_H_

#include "apstring.h"
#include "apvector.h"

struct LNODE {              // Node of the Morse Code encoding list
    char symbol;
    apstring code;
    LNODE *next;

    LNODE(char c, const apstring &str)
        : symbol(c), code(str), next(0) {}       // Constructor
};

struct TNODE {              // Node of the Morse Code decoding tree
    char symbol;
    TNODE *left;
    TNODE *right;

    TNODE() : symbol('*'), left(0), right(0) {}  // Constructor
};

class TELEGRAPH {

    // This is an unusual class: it has only static data members--
    //   mHead, the head of the encoding list, and mRoot, the root
    //   of the decoding tree.  These list and tree are the same for all
    //   class instances.
    //
    //   Such class doesn't need a constructor or destructor because they
    //   wouldn't be able to deal with static members, anyway.  Instead
    //   the list and the tree are initialized in the static member
    //   function Open() and destroyed in the static function Close().

  public:

    static void Open();
    static void Close();

    void Encode(const apstring &text, apstring &morse);
    void Decode(const apstring &morse, apstring &text);

  private:

    static void RecursiveDestroy(TNODE *root);
    static void Insert(char symbol, const apstring &morsecode);

    static LNODE *mHead;        // The head of the decoding list.
    static TNODE *mRoot;        // The root of the decoding tree.
};

#endif // _MORSE_H_

