// MUSIC.CPP
//
// This program displays the lyrics of a song written in a file
// with embedded #repeat commands.
//
// Author: J. Lennon and P. McCartney
//

#include <iostream.h>
#include <fstream.h>
#include <strstream.h>  // Some compilers use <strstrea.h>
#include "apstring.h"
#include "apvector.h" 
#include "apstack.h"

void ShowTextSimple (const apvector<apstring> &text,
                                       int fromLine, int toLine);
void Parse (const apstring &line, int &fromLine, int &toLine);
void ShowText (const apvector<apstring> &text, int fromLine, int toLine);
                                                                               
//****************************************************************

int main()

{
    const int MAXLINES = 1000;
    apvector<apstring> text(MAXLINES);
    apstring fileName;
    int nLines = 0;

    // Prompt the user for a file name and open the file.
    // If no extension given, append the default extension ".txt":
    cout << "Text file name: ";
    cin >> fileName;
    if (fileName.find('.') == npos)
        fileName += ".txt";

    ifstream textFile(fileName.c_str());
    if (!textFile) {
        cout << "Cannot open " << fileName << ".\n";
        return 1;
    }

    // Read the lines from the file into an array of strings:
    while (nLines < MAXLINES && getline(textFile, text[nLines]))
        nLines++;

    // Show the text "as is":
    ShowTextSimple(text, 1, nLines);

    cout << "\n************************************\n\n";

    // Show the text with correctly processed #repeat commands:
    ShowText(text, 1, nLines);           

    return 0;
}

//****************************************************************

void ShowTextSimple (const apvector<apstring> &text,
                                       int fromLine, int toLine)

// Displays lines of text from fromLine to toLine.
// Lines are numbered starting from 1.

{
    while (fromLine <= toLine) {
        cout << text[fromLine-1] << endl;
        fromLine++;
    }
}

//****************************************************************

void ShowText (const apvector<apstring> &text, int fromLine, int toLine)

// Displays lines of text from fromLine to toLine.
// Handles #repeat directives embedded in the text:
// #repeat fromNum toNum
//   (the first character in the line must be #; fromNum and toNum
//    are integers).
// Lines are numbered starting from 1.

{
    apstack<int> stack;
    apstring line;

    while (fromLine <= toLine || !stack.isEmpty()) {
        if (fromLine <= toLine) {
            line = text[fromLine-1];
            if (line.length() == 0 || line[0] != '#') {
                // Regular line
                cout << line << endl;
                fromLine++;
            }
            else {
                // #repeat line
                ...
                ...
                Parse(line, fromLine, toLine);
            }
        }
        else { // if (!stack.isEmpty())
            ...
            ...
        }
    }
}

//****************************************************************

void Parse (const apstring &line, int &fromLine, int &toLine)

// Parses a repeat directive
// #repeat fromNum toNum
// extracts fromNum and toNum and places them into fromLine, toLine.

{
    apstring discard;
    istrstream is( const_cast<char *>(line.c_str()) );
         // Creates an input stream associated with the string.
         // istrstream constructor takes one argument of the char* type.
         // c_str() converts apstring into const char*
         // const_cast converts const char* into char*

         // In older compilers use C-style cast:
         //    istrstream is( (char *)(line.c_str()) );

        is >> discard >> fromLine >> toLine;
}

