// RBQUEUE.CPP
//
// Queue implemented as a ring buffer.

#include "rbqueue.h"

RBQUEUE::RBQUEUE(int size)

// Constructor

{
    mBuffer = new char[size];
    if (mBuffer)
        mSize = size;
    else
        mSize = 0;

    mFront = 0;
    mRear = 0;
}

//****************************************************************

RBQUEUE::~RBQUEUE()

// Destructor

{
    delete [] mBuffer;
}

//****************************************************************

void RBQUEUE::enqueue (char c)

// Appends c at the end of the buffer.

{
    int i = NextIndex(mRear);
    if (i == mFront)            // The queue is full
        return;
    mBuffer[mRear] = c;
    mRear = i;
}

//****************************************************************

void RBQUEUE::dequeue (char &c)

// Retrieves and removes the element from the front of the buffer.

{
    if (mFront == mRear)        // The queue is empty
        return;
    c = mBuffer[mFront];
    mFront = NextIndex(mFront);
}

bool RBQUEUE::isEmpty()

// Returns true if the queue is empty, false otherwise.

{
    return mFront == mRear;
}

int RBQUEUE::NextIndex(int index)

// Calculates and returns the value of the next index
//   with wraparound.

{
    index++;
    if (index == mSize)
        index = 0;

    return index;
}
