// TREE.CPP

#include "tree.h"

//****************************************************************

void Destroy(TREENODE *root)

{
    // Base case 0: the tree is empty -- do nothing;
    if (root) {  // Recursive case:
        Destroy(root->left);
        Destroy(root->right);
        delete root;
    }
}

//****************************************************************
#ifdef RECURSIVE

TREENODE *Find(TREENODE *root, const apstring &target)

// Finds the node such that node->data == target.
// Returns the pointer to the found node, or 0 if not found.

{
    if (!root)     // Base case 0: the tree is empty.
        return 0;

    if (target == root->data)
        return root;

    else if (target < root->data)         // Recursive case: search
        return Find(root->left, target);  //   left subtree.

    else // if (target > root->data)      // Recursive case: search
        return Find(root->right, target); //   right subtree.
}

#else

TREENODE *Find(TREENODE *root, const apstring &target)

// *** Iterative version ***

{
    while (root && target != root->data) {
        if (target < root->data)
            root = root->left;
        else // if (target > root->data)
            root = root->right;
    }
    return root;
}

#endif

//****************************************************************

bool Insert(TREENODE* &root, const apstring &data)

// Creates a new node with node->data = data and inserts
//   it into the proper place in the tree.
// Returns true if successful, false if cannot
//    allocate a new node.

{
    if (!root) { // Base case 0: the tree is empty.

        // Allocate a new node and set the root
        //   equal to the new node:

        TREENODE *newnode = new TREENODE;
        if (!newnode)
            return false;
        newnode->data = data;
        newnode->left = 0;
        newnode->right = 0;
        root = newnode;
        return true;
    }

    if (data == root->data) // Base case: the element is already
                            //   in the tree.
        return true;        //   (Another version might report an error)

    else if (data < root->data)           // Recursive case: insert
        return Insert(root->left, data);  //   into the left subtree

    else // if (data > root->data)        // Recursive case: insert
        return Insert(root->right, data); //   into the right subtree
}

//****************************************************************

bool Remove(TREENODE* &root, const apstring &data)

// Removes the node whose data element is equal to data.
// Returns true if successful, false if cannot
//    find the node.

{
    if (!root) // Base case 0: the tree is empty.
        return false;

    if (data == root->data) {
        // Base case:
        //   found the node. root refers to
        //   the pointer to the node to be removed...

        // Case 1: root has no more than one child. Replace root
        //   pointer with the pointer to that child.
        //   That unlinks the root node from the tree.
        //   Then delete the node.

        // First save root -- it points to the
        //   node to be removed:

        TREENODE *oldroot = root;

        if (root->left == 0)       // No left child
            root = root->right;
        else if (root->right == 0) // No right child
            root = root->left;
        if (oldroot != root) {     // If one of the above...
            delete oldroot;
            return true;
        }

        // Case 2: root has both children.
        //   Find the smallest element 
        //   in the right subtree.  Unlink that element and
        //   make it the new root.

        // First find the smallest element in the right subtree.
        //   To do that, keep going left as long as possible.
        //   Also keep track of its parent

        TREENODE *parent = root;
        TREENODE *next = root->right;
        while (next->left) {
            parent = next;
            next = next->left;
        }

        // Unlink this node from the tree (it doesn't have
        //   a left child, so we can connect its right child to its
        //   parent)

        if (parent == root)
            root->right = next->right;
        else
            parent->left = next->right;

        // Make this element the new root

        next->left = root->left;
        next->right = root->right;
        delete root;
        root = next;

        return true;
    }

    else if (data < root->data)           // Recursive case: remove
        return Remove(root->left, data);  //   from the left subtree

    else // if (data > root->data)        // Recursive case: remove
        return Remove(root->right, data); //   from the right subtree
}

