// TREE.H
// ======

#ifndef _TREE_H_
#define _TREE_H_

#include "bool.h"
#include "apstring.h"

struct TREENODE {
    apstring data;
    TREENODE *left;
    TREENODE *right;
};

typedef void CALLBACK(TREENODE *node);
    // Defines the CALLBACK data type as a function which takes one
    //   argument of the type TREENODE *...

class TREE {

  public:

    TREE() {       // Default constructor
        mRoot = 0;
    }

    bool Insert(const apstring &data) {
        return RecursiveInsert(mRoot, data);
    }

    void TraverseInOrder(CALLBACK Visit) {
        RecursiveTraverseInOrder(mRoot, Visit);
    }
//    ...
    ~TREE() {
        Destroy(mRoot);
    }

  private:

    TREENODE *mRoot;

    // recursive "helper" functions:
    bool RecursiveInsert(TREENODE* &root, const apstring &data);
    void RecursiveTraverseInOrder(TREENODE *root, CALLBACK Visit);
//    ...
    void Destroy(TREENODE *root);

};

#endif // _TREE_H_

// TREE.CPP
// ========

// #include "tree.h"

bool TREE::RecursiveInsert(TREENODE* &root, const apstring &data)

{
    if (!root) { // Base case 0: the tree is empty.

        // Allocate a new node and set the root
        //   equal to the new node:

        TREENODE *newnode = new TREENODE;
        if (!newnode)
            return false;
        newnode->data = data;
        newnode->left =
        newnode->right = 0;
        root = newnode;
        return true;
    }

    if (data == root->data)
        return true;
    else if (data < root->data)
        return RecursiveInsert(root->left, data);
    else // if (data > root->data)
        return RecursiveInsert(root->right, data);
}

void TREE::RecursiveTraverseInOrder(TREENODE *root, CALLBACK Visit)

{
    if (!root) return;
    RecursiveTraverseInOrder(root->left, Visit);
    Visit(root);
    RecursiveTraverseInOrder(root->right, Visit);
}

// ...

void TREE::Destroy(TREENODE *root)

{
    if (root) {
        Destroy(root->left);
        Destroy(root->right);
        delete root;
    }
}

// TEST.CPP
// ========

#include <iostream.h>
//#include "tree.h"

static void Visit (TREENODE *node)

{
    cout << root->node << ' ';
}

int main()

{
    TREE tree;

    tree.Insert("Mexico City");
    tree.Insert("Munich");
    tree.Insert("Montreal");
    tree.Insert("Moscow");
    tree.Insert("Los Angeles");
    tree.Insert("Seoul");
    tree.Insert("Barcelona");
    tree.Insert("Atlanta");
    tree.Insert("Sydney");

    cout << "Inorder:\n";
    tree.TraverseInOrder(Visit);
    cout << endl;
    return 0;
}


