// TREEFILE.CPP

#include <iostream.h>
#include <fstream.h>
#include "tree.h"

static void RecursiveSave (ofstream &file, TREENODE *root);
static void RecursiveLoad (ifstream &file, TREENODE* &root);

//****************************************************************

void SaveTree (const apstring &fileName, TREENODE *root)

// Opens a file for writing and saves the tree in it.

{
    ofstream file(fileName.c_str());
    if (!file) {
        cerr << "Cannot create " << fileName << endl;
        return;
    }
    RecursiveSave(file, root);
}

//****************************************************************

void LoadTree (const apstring &fileName, TREENODE* &root)

// Opens a file for reading and loads the tree from it.
// If the file does not exist, returns null.

{
    ifstream file(fileName.c_str());
    if (!file) {
        cerr << "Cannot open " << fileName << endl;
        return;
    }
    RecursiveLoad(file, root);
}

//****************************************************************
//***         Recursive Save and Load functions                ***
//****************************************************************

static void RecursiveSave (ofstream &file, TREENODE *root)

// Recursive helper function -- called from SaveTree;
// Assumes that file is open for writing.

{
    file << "[";
    if (root) {
        file << endl;
        file << root->data << endl;
        RecursiveSave (file, root->left);
        RecursiveSave (file, root->right);
    }
    file << "]\n";  // An empty tree is saved as one line: []
}

//****************************************************************

static void RecursiveLoad (ifstream &file, TREENODE* &root)

// Recursive helper function -- called from LoadTree.
// Assumes that file is open for reading and that the tree is empty.

{
    apstring line;
              
    if (root != 0)
        return;

    // Skip all lines before the first '[':
    while (getline(file, line))
        if (line.length() > 0 && line[0] == '[')
             break;

    if (line.length() >= 2 && line[1] == ']')
                                      // If the tree is empty...
        return;

    getline(file, line);              // Read the next line
    root = new TREENODE;
    root->data = line;
    root->left =
    root->right = 0;
    RecursiveLoad(file, root->left);
    RecursiveLoad(file, root->right);
    getline(file, line);              // Read the closing ']'
    if (line.length() < 1 || line[0] != ']')
        cerr << "Tree file format error: " << line << endl;
}

// TEST.CPP
// ========

inline void Visit (TREENODE *node)

{
    cout << node->data << ' ';
}

void TraverseInOrder(TREENODE *root)

{
    if (!root) return;
    TraverseInOrder(root->left);
    Visit(root);
    TraverseInOrder(root->right);
}

//****************************************************************

int main()

{
    TREENODE *root = 0;

    Insert(root, "Mexico City");
    Insert(root, "Munich");
    Insert(root, "Montreal");
    Insert(root, "Moscow");
    Insert(root, "Los Angeles");
    Insert(root, "Seoul");
    Insert(root, "Barcelona");
    Insert(root, "Atlanta");
    Insert(root, "Sydney");

    TraverseInOrder(root);
    cout << "\n";
    SaveTree("tree.dat", root);
    Destroy(root);
    root = 0;
    LoadTree("tree.dat", root);
    TraverseInOrder(root);
    cout << "\n";
    Destroy(root);
    return 0;
}

